#include "csapp.h"
#include "config.h"

/* This file demonstrates a way to run the untrusted functions so that
   they can't get at the other half of the secret or the other
   checker's result. */

void compute_initial_secrets(int *secret1, int *secret2);
void adjust_secrets(int *secret1, int *secret2, int i);
    
void check1(int *secret1, int *answer1, int i);
void check2(int *secret2, int *answer2, int i);

static int *alloc(int opt) {
  return mmap(NULL, SIZE * sizeof(int),
              PROT_READ | PROT_WRITE, opt | MAP_ANON,
              -1, 0);
}

static void dealloc(void *p) {
  munmap(p, SIZE * sizeof(int));
}

int main()
{
  int *secret1 = alloc(MAP_PRIVATE);
  int *secret2 = alloc(MAP_PRIVATE);
  int *answer1 = alloc(MAP_SHARED);
  int *answer2 = alloc(MAP_SHARED);
  int i;
  pid_t pid1, pid2;

  compute_initial_secrets(secret1, secret2);

  for (i = 0; i < ITERS; i++) {
    adjust_secrets(secret1, secret2, i);

    pid1 = Fork();
    if (pid1 == 0) {
      dealloc(secret2);
      dealloc(answer2);
      check1(secret1, answer1, i);
      exit(0);
    }

    pid2 = Fork();
    if (pid2 == 0) {
      dealloc(secret1);
      dealloc(answer1);
      check2(secret2, answer2, i);
      exit(0);
    }

    Waitpid(pid1, NULL, 0);
    Waitpid(pid2, NULL, 0);
    
    if (memcmp(answer1, answer2, SIZE * sizeof(int))) {
      printf("disagree %d\n", i);
      exit(1);
    }
  }

  printf("ok\n");
  return 0;
}
