#include "csapp.h"
#include "config.h"

/* This file is like "direct.c", but demonstrates that running check1
   and check2 in separate processes and piping back the results would
   be way too slow. */

void compute_initial_secrets(int *secret1, int *secret2);
void adjust_secrets(int *secret1, int *secret2, int i);
    
void check1(int *secret1, int *answer1, int i);
void check2(int *secret2, int *answer2, int i);

static int *alloc() {
  return malloc(SIZE * sizeof(int));
}

static void erase(void *p) {
  memset(p, 0, SIZE * sizeof(int));
}

int main()
{
  int *secret1 = alloc();
  int *secret2 = alloc();
  int *answer1 = alloc();
  int *answer2 = alloc();
  int i, fds1[2], fds2[2];
  pid_t pid1, pid2;

  compute_initial_secrets(secret1, secret2);

  for (i = 0; i < ITERS; i++) {
    adjust_secrets(secret1, secret2, i);

    Pipe(fds1);
    pid1 = Fork();
    if (pid1 == 0) {
      Close(fds1[0]);
      erase(secret2);
      erase(answer2);
      check1(secret1, answer1, i);
      rio_writen(fds1[1], answer1, SIZE * sizeof(int));
      exit(0);
    }

    Close(fds1[1]);

    Pipe(fds2);
    pid2 = Fork();
    if (pid2 == 0) {
      Close(fds1[0]);
      Close(fds2[0]);
      erase(secret1);
      erase(answer1);
      check2(secret2, answer2, i);
      rio_writen(fds2[1], answer2, SIZE * sizeof(int));
      exit(0);
    }

    Close(fds2[1]);

    rio_readn(fds1[0], answer1, SIZE * sizeof(int));
    rio_readn(fds2[0], answer2, SIZE * sizeof(int));

    Close(fds1[0]);
    Close(fds2[0]);
    
    Waitpid(pid1, NULL, 0);
    Waitpid(pid2, NULL, 0);

    if (memcmp(answer1, answer2, SIZE * sizeof(int))) {
      printf("disagree %d\n", i);
      exit(1);
    }
  }

  printf("ok\n");
  return 0;
}
